from django import forms
from django.contrib.auth.models import User, Group
from .models import Clientes, Transacciones, Vehiculos, Importaciones
from django.contrib.auth.forms import PasswordChangeForm

#Cliente
class ClienteForm(forms.ModelForm):
    class Meta:
        model = Clientes
        fields = ['nombre', 'telefono', 'direccion', 'correo', 'saldo_actual']
        widgets = {
            'nombre': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Nombre'}),
            'telefono': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Teléfono'}),
            'direccion': forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Dirección'}),
            'correo': forms.EmailInput(attrs={'class': 'form-control', 'placeholder': 'Correo'}),
            'saldo_actual': forms.NumberInput(attrs={'class': 'form-control', 'placeholder': 'Saldo Actual'}),
        }

#Usuario
class UsuarioForm(forms.ModelForm):
    password = forms.CharField(
        label="Contraseña",
        widget=forms.PasswordInput(attrs={'class': 'form-control'})
    )
    confirm_password = forms.CharField(
        label="Confirmar Contraseña",
        widget=forms.PasswordInput(attrs={'class': 'form-control'})
    )
    groups = forms.ModelChoiceField(
        queryset=Group.objects.all(),
        required=True,
        label="Rol",
        widget=forms.Select(attrs={'class': 'form-control'}),
    )

    class Meta:
        model = User
        fields = ['username', 'email', 'first_name', 'last_name', 'password', 'groups']
        widgets = {
            'username': forms.TextInput(attrs={'class': 'form-control'}),
            'email': forms.EmailInput(attrs={'class': 'form-control'}),
            'first_name': forms.TextInput(attrs={'class': 'form-control'}),
            'last_name': forms.TextInput(attrs={'class': 'form-control'}),
        }

    def clean(self):
        cleaned_data = super().clean()
        password = cleaned_data.get('password')
        confirm_password = cleaned_data.get('confirm_password')

        if password != confirm_password:
            raise forms.ValidationError("Las contraseñas no coinciden.")
        return cleaned_data

    def save(self, commit=True):
        # Sobrescribe el método save para encriptar la contraseña
        user = super().save(commit=False)
        user.set_password(self.cleaned_data['password'])  # Encripta la contraseña
        if commit:
            user.save()  # Guarda el usuario
            # Asigna el grupo al usuario
            grupo = self.cleaned_data.get('groups')
            if grupo:
                user.groups.add(grupo)
        return user
    
class EditarUsuarioForm(forms.ModelForm):
    grupos = forms.ModelMultipleChoiceField(
        queryset=Group.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        required=False,
        label="Roles"
    )

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']

#Transaccion
class TransaccionForm(forms.ModelForm):
    class Meta:
        model = Transacciones
        fields = ['fecha', 'monto_recibido', 'monto_debitado', 'concepto', 'proveedor']  # Excluye 'cliente'
        widgets = {
            'fecha': forms.DateInput(attrs={'type': 'date'}),
            'monto_recibido': forms.NumberInput(attrs={'step': '0.01'}),
            'monto_debitado': forms.NumberInput(attrs={'step': '0.01'}),
            'concepto': forms.TextInput(attrs={'placeholder': 'Concepto'}),
            'proveedor': forms.TextInput(attrs={'placeholder': 'Proveedor'}),
        }

#Vehiculos
class VehiculoForm(forms.ModelForm):
    class Meta:
        model = Vehiculos
        fields = ['marca', 'modelo', 'anho', 'nro_chasis', 'numero_despacho', 'estado', 'cliente']
        widgets = {
            'marca': forms.TextInput(attrs={'class': 'form-control'}),
            'modelo': forms.TextInput(attrs={'class': 'form-control'}),
            'anho': forms.NumberInput(attrs={'class': 'form-control'}),
            'nro_chasis': forms.TextInput(attrs={'class': 'form-control'}),
            'estado': forms.Select(attrs={'class': 'form-select'}),
            'cliente': forms.Select(attrs={'class': 'form-select'}),
            'numero_despacho': forms.Select(attrs={'class': 'form-select'}),
        }
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Personalizar el desplegable de 'numero_despacho'
        self.fields['numero_despacho'].queryset = Importaciones.objects.all()
        self.fields['numero_despacho'].label_from_instance = lambda obj: (
            f"{obj.despacho} - Espacio disponible: {obj.espacio_disponible()}"
        )

    def clean_numero_despacho(self):
        numero_despacho = self.cleaned_data['numero_despacho']
        if numero_despacho.espacio_disponible() <= 0:
            raise forms.ValidationError(
                f"El despacho {numero_despacho.despacho} ya tiene todos sus autos asignados."
            )
        return numero_despacho

#Importaciones 
class ImportacionesForm(forms.ModelForm):
    class Meta:
        model = Importaciones
        fields = ['despacho', 'fecha', 'despachante', 'proveedor', 'item', 'aduana', 'cliente', 'facturado', 'cobrado']
        widgets = {
            'despacho': forms.TextInput(attrs={'class': 'form-control'}),
            'fecha': forms.DateInput(attrs={'type': 'date', 'class': 'form-control'}),
            'despachante': forms.TextInput(attrs={'class': 'form-control'}),  # Cambiado a TextInput
            'proveedor': forms.TextInput(attrs={'class': 'form-control'}),
            'item': forms.NumberInput(attrs={'class': 'form-control'}),
            'aduana': forms.TextInput(attrs={'class': 'form-control'}),
            'cliente': forms.Select(attrs={'class': 'form-select'}),  # Correcto para ForeignKey
            'facturado': forms.CheckboxInput(attrs={'class': 'form-check-input'}),  # BooleanField
            'cobrado': forms.CheckboxInput(attrs={'class': 'form-check-input'}),  # BooleanField
        }

#Cambio de contraseña 
class CustomPasswordChangeForm(PasswordChangeForm):
    old_password = forms.CharField(
        widget=forms.PasswordInput(attrs={'class': 'form-control', 'placeholder': 'Contraseña actual'}),
        label="Contraseña Actual"
    )
    new_password1 = forms.CharField(
        widget=forms.PasswordInput(attrs={'class': 'form-control', 'placeholder': 'Nueva contraseña'}),
        label="Nueva Contraseña"
    )
    new_password2 = forms.CharField(
        widget=forms.PasswordInput(attrs={'class': 'form-control', 'placeholder': 'Confirmar nueva contraseña'}),
        label="Confirmar Nueva Contraseña"
    )

    def clean_new_password1(self):
        new_password1 = self.cleaned_data.get("new_password1")
        if len(new_password1) < 6:  # Validación básica
            raise forms.ValidationError("La nueva contraseña debe tener al menos 6 caracteres.")
        return new_password1