from django.db import models
from django.forms import ValidationError


class Clientes(models.Model):
    id_cliente = models.AutoField(primary_key=True)
    nombre = models.CharField(max_length=255)
    telefono = models.CharField(max_length=50, blank=True, null=True)
    direccion = models.CharField(max_length=255, blank=True, null=True)
    correo = models.EmailField(blank=True, null=True)
    saldo_actual = models.DecimalField(max_digits=10, decimal_places=2, default=0)

    def __str__(self):
        return self.nombre


class Transacciones(models.Model):
    id_transaccion = models.AutoField(primary_key=True)
    cliente = models.ForeignKey(Clientes, on_delete=models.CASCADE, related_name="transacciones")
    fecha = models.DateField()
    monto_recibido = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True, default=0)
    monto_debitado = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True, default=0)
    saldo_post_transaccion = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    concepto = models.TextField(blank=True, null=True)
    proveedor = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        return f"{self.cliente.nombre} - {self.fecha}"


class Vehiculos(models.Model):
    id_vehiculo = models.AutoField(primary_key=True)
    marca = models.CharField(max_length=50)
    modelo = models.CharField(max_length=50, blank=True, null=True)
    anho = models.IntegerField(blank=True, null=True)
    nro_chasis = models.CharField(max_length=50, unique=True)
    numero_despacho = models.ForeignKey(
        'Importaciones', 
        on_delete=models.SET_NULL, 
        null=True, 
        blank=True, 
        related_name='vehiculos'
    )
    estado = models.CharField(
        max_length=50,
        choices=[
            ("Pendiente de venta", "Pendiente de venta"),
            ("Vendido", "Vendido"),
            ("Transferencia pendiente", "Transferencia pendiente"),
            ("Transferido", "Transferido"),
        ],
        default="Pendiente de venta",
    )
    cliente = models.ForeignKey(
        'Clientes', 
        on_delete=models.SET_NULL, 
        null=True, 
        blank=True, 
        related_name='vehiculos'  # Este nombre se usará para acceder desde Clientes
    )

    def __str__(self):
        return f"{self.marca} {self.modelo} ({self.nro_chasis})"



class Ventas(models.Model):
    id_venta = models.AutoField(primary_key=True)
    vehiculo = models.ForeignKey(Vehiculos, on_delete=models.CASCADE)
    cliente = models.ForeignKey(Clientes, on_delete=models.CASCADE)
    fecha_venta = models.DateField()
    monto_total = models.DecimalField(max_digits=10, decimal_places=2)
    monto_pagado = models.DecimalField(max_digits=10, decimal_places=2)
    estado_transferencia = models.CharField(
        max_length=50,
        choices=[
            ("Pendiente", "Pendiente"),
            ("Completada", "Completada"),
        ],
        default="Pendiente",
    )
    adeuda = models.DecimalField(max_digits=10, decimal_places=2, default=0)

    def __str__(self):
        return f"Venta {self.id_venta} - {self.cliente.nombre}"


class Cobros(models.Model):
    id_cobro = models.AutoField(primary_key=True)
    cliente = models.ForeignKey(Clientes, on_delete=models.CASCADE)
    fecha = models.DateField()
    monto_usd = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    monto_gs = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)
    cantidad_autos = models.PositiveIntegerField(default=0)  # Nuevo campo
    monto_pagar_usd = models.DecimalField(max_digits=10, decimal_places=2, blank=True, null=True)  # Nuevo campo
    observaciones = models.TextField(blank=True, null=True)

    def __str__(self):
        return f"Cobro de {self.cliente.nombre} - {self.fecha}"

class Importaciones(models.Model):
    id_importacion = models.AutoField(primary_key=True)
    despacho = models.CharField(max_length=100, unique=True)
    fecha = models.DateField()
    despachante = models.CharField(max_length=255)
    proveedor = models.CharField(max_length=255)
    item = models.IntegerField()  # Número de autos permitidos
    aduana = models.CharField(max_length=255)
    cliente = models.ForeignKey('Clientes', on_delete=models.CASCADE, related_name='importaciones')
    facturado = models.BooleanField(default=False)
    cobrado = models.BooleanField(default=False)

    def autos_asociados(self):
        # Contar los autos relacionados con este despacho
        return self.vehiculos.count()

    def espacio_disponible(self):
        # Verificar si se pueden asociar más autos
        return self.item - self.autos_asociados()

    def __str__(self):
        return f"Despacho {self.despacho} - {self.proveedor} - {self.fecha}"