from django.contrib import messages
from datetime import datetime
from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required
from .models import Clientes, Vehiculos, Transacciones, Cobros, Importaciones
from .forms import UsuarioForm, ClienteForm, TransaccionForm, VehiculoForm, EditarUsuarioForm, ImportacionesForm, CustomPasswordChangeForm 
from django.core.paginator import Paginator
from django.db.models import Sum
from django.db.models import Count
from core.utils import verificar_permisos
from .permissions import es_administrador, es_vendedor, es_visitante
from django.contrib.auth import update_session_auth_hash

#Dashboard principal
@login_required
def dashboard(request):
    user = request.user
    is_admin = user.groups.filter(name="Administrador").exists()
    is_seller = user.groups.filter(name="Vendedor").exists()
    is_visitor = user.groups.filter(name="Visitante").exists()
    total_vehiculos = Vehiculos.objects.count()
    vehiculos_por_estado = Vehiculos.objects.values('estado').annotate(cantidad=Count('estado'))
    vehiculos_por_estado_dict = {item['estado']: item['cantidad'] for item in vehiculos_por_estado}
    return render(request, 'core/dashboard.html', {
        'is_admin': is_admin,
        'is_seller': is_seller,
        'is_visitor': is_visitor,
        'total_vehiculos': total_vehiculos,
        'vehiculos_por_estado': vehiculos_por_estado_dict,
    })

#Usuario
@login_required
@verificar_permisos(lambda user: user.is_superuser)  # Solo para administradores
def detalle_usuario(request, usuario_id):
    usuario = get_object_or_404(User, id=usuario_id)
    
    # Determinar el rol basado en los grupos
    grupos = usuario.groups.all()
    if grupos.exists():
        rol = ", ".join(grupo.name for grupo in grupos)
    else:
        rol = "No asignado"
    
    return render(request, 'core/detalle_usuario.html', {
        'usuario': usuario,
        'rol': rol,
    })

@login_required
@verificar_permisos(es_administrador)
def crear_usuario(request):
    if request.method == 'POST':
        form = UsuarioForm(request.POST)
        if form.is_valid():
            usuario = form.save(commit=False)
            usuario.save()
            # Asignar grupo
            grupo = form.cleaned_data['groups']
            if grupo:
                usuario.groups.add(grupo)
            return redirect('listar_usuarios')
    else:
        form = UsuarioForm()

    return render(request, 'core/crear_usuario.html', {'form': form})

@login_required
@verificar_permisos(es_administrador)
def listar_usuarios(request):
    usuarios_list = User.objects.all().order_by('id')  # Ordena explícitamente por el campo 'id'
    paginator = Paginator(usuarios_list, 10)  # Mostrar 10 usuarios por página
    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)
    return render(request, 'core/listar_usuarios.html', {'page_obj': page_obj})

@login_required
@verificar_permisos(lambda user: user.is_superuser)  # Solo para administradores
def editar_usuario(request, usuario_id):
    usuario = get_object_or_404(User, id=usuario_id)
    
    if request.method == 'POST':
        form = EditarUsuarioForm(request.POST, instance=usuario)
        if form.is_valid():
            usuario = form.save(commit=False)
            
            # Actualizar grupos (roles)
            grupos = form.cleaned_data['grupos']
            usuario.groups.set(grupos)
            usuario.save()

            return redirect('detalle_usuario', usuario_id=usuario.id)
    else:
        form = EditarUsuarioForm(instance=usuario, initial={
            'grupos': usuario.groups.all(),
        })

    return render(request, 'core/editar_usuario.html', {'form': form, 'usuario': usuario})

@login_required
@verificar_permisos(lambda user: user.is_superuser)  # Solo para administradores
def eliminar_usuario(request, user_id):
    usuario = get_object_or_404(User, id=user_id)
    if request.method == 'POST':
        usuario.delete()
        messages.success(request, f'El usuario {usuario.username} ha sido eliminado correctamente.')
        return redirect('listar_usuarios')  # Asegúrate de tener esta URL configurada.
    
    return render(request, 'core/eliminar_usuario.html', {'usuario': usuario})


#Seccion de Clientes
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user))
def crear_cliente(request):
    if request.method == 'POST':
        form = ClienteForm(request.POST)
        if form.is_valid():  # Validar datos
            form.save()  # Guardar automáticamente en la base de datos
            return redirect('listar_clientes')  # Redirigir después de guardar
    else:
        form = ClienteForm()
    return render(request, 'core/crear_cliente.html', {'form': form})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user) or es_visitante(user))
def listar_clientes(request):
    clientes = Clientes.objects.all()
    paginator = Paginator(clientes, 10)  # 10 clientes por página
    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)
    return render(request, 'core/listar_clientes.html', {'page_obj': page_obj})

@login_required
@verificar_permisos(es_administrador)
def editar_cliente(request, cliente_id):
    cliente = get_object_or_404(Clientes, id_cliente=cliente_id)  # Cambiar 'id' por 'id_cliente'
    if request.method == 'POST':
        cliente.nombre = request.POST['nombre']
        cliente.telefono = request.POST['telefono']
        cliente.direccion = request.POST['direccion']
        cliente.correo = request.POST['correo']
        cliente.saldo_actual = request.POST.get('saldo_actual', 0)
        cliente.save()
        return redirect('listar_clientes')

    return render(request, 'core/editar_cliente.html', {'cliente': cliente})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def detalle_cliente(request, cliente_id):
    # Obtener el cliente
    cliente = get_object_or_404(Clientes, id_cliente=cliente_id)
    # Obtener mes y año desde el formulario GET, o valores por defecto (mes y año actuales)
    mes_actual = int(request.GET.get('mes', datetime.now().month))
    anho_actual = int(request.GET.get('anho', datetime.now().year))
    # Diccionario para nombres de meses en español
    meses_espanol = {
        1: "Enero", 2: "Febrero", 3: "Marzo", 4: "Abril",
        5: "Mayo", 6: "Junio", 7: "Julio", 8: "Agosto",
        9: "Septiembre", 10: "Octubre", 11: "Noviembre", 12: "Diciembre"
    }
    transacciones = cliente.transacciones.filter(
        fecha__month=mes_actual,
        fecha__year=anho_actual
    )
    vehiculos = cliente.vehiculos.filter(
        numero_despacho__fecha__month=mes_actual,
        numero_despacho__fecha__year=anho_actual
    )
    cobros = Cobros.objects.filter(
        cliente=cliente,
        fecha__month=mes_actual,
        fecha__year=anho_actual
    )
    nombre_mes = meses_espanol[mes_actual]
    return render(request, 'core/detalle_cliente.html', {
        'cliente': cliente,
        'transacciones': transacciones,
        'vehiculos': vehiculos,
        'cobros': cobros,
        'nombre_mes': nombre_mes,
        'anho_actual': anho_actual,
        'meses_espanol': meses_espanol,
        'mes_seleccionado': mes_actual,
    })

@login_required
@verificar_permisos(es_administrador)
def eliminar_cliente(request, cliente_id):
    cliente = get_object_or_404(Clientes, id_cliente=cliente_id)  # Cambiar 'id' por 'id_cliente'
    cliente.delete()
    return redirect('listar_clientes')

# Seccion de Vehiculos
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user))
def listar_vehiculos(request):
    vehiculos = Vehiculos.objects.all().order_by("id_vehiculo")
    paginator = Paginator(vehiculos, 10)  # Mostrar 10 vehículos por página
    page_number = request.GET.get("page")
    page_obj = paginator.get_page(page_number)

    return render(request, "core/listar_vehiculos.html", {
        "page_obj": page_obj,
    })

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def crear_vehiculo(request):
    if request.method == "POST":
        form = VehiculoForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, "Vehículo registrado con éxito.")
            return redirect('listar_vehiculos')
    else:
        form = VehiculoForm()
    return render(request, 'core/crear_vehiculo.html', {'form': form})

@login_required
@verificar_permisos(es_administrador)
def editar_vehiculo(request, vehiculo_id):
    vehiculo = get_object_or_404(Vehiculos, id_vehiculo=vehiculo_id)
    if request.method == "POST":
        form = VehiculoForm(request.POST, instance=vehiculo)
        if form.is_valid():
            form.save()
            messages.success(request, "Vehículo actualizado con éxito.")
            return redirect('listar_vehiculos')
    else:
        form = VehiculoForm(instance=vehiculo)
    return render(request, 'core/editar_vehiculo.html', {'form': form})

@login_required
@verificar_permisos(es_administrador)
def eliminar_vehiculo(request, vehiculo_id):
    vehiculo = get_object_or_404(Vehiculos, id_vehiculo=vehiculo_id)
    if request.method == 'POST':
        vehiculo.delete()
        return redirect('listar_vehiculos')
    return render(request, 'core/eliminar_vehiculo.html', {'vehiculo': vehiculo})

#Seccion Transacciones
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user))
def listar_transacciones(request):
    transacciones = Transacciones.objects.all().order_by('-fecha')  # Ordenar por fecha descendente
    paginator = Paginator(transacciones, 10)  # Mostrar 10 transacciones por página

    page_number = request.GET.get('page')  # Obtener el número de página de la solicitud GET
    page_obj = paginator.get_page(page_number)  # Obtener la página solicitada

    return render(request, 'core/listar_transacciones.html', {'page_obj': page_obj})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def crear_transaccion(request, cliente_id):
    # Obtener el cliente específico usando el cliente_id
    cliente = get_object_or_404(Clientes, id_cliente=cliente_id)

    if request.method == 'POST':
        form = TransaccionForm(request.POST)
        if form.is_valid():
            transaccion = form.save(commit=False)
            transaccion.cliente = cliente  # Asignar el cliente automáticamente
            
            # Actualizar el saldo del cliente
            if transaccion.monto_recibido:
                cliente.saldo_actual += transaccion.monto_recibido
            if transaccion.monto_debitado:
                cliente.saldo_actual -= transaccion.monto_debitado
            
            cliente.save()  # Guardar cliente con saldo actualizado
            transaccion.save()  # Guardar transacción
            
            messages.success(request, 'Transacción registrada y saldo actualizado con éxito.')
            return redirect('detalle_cliente', cliente_id=cliente.id_cliente)
    else:
        form = TransaccionForm()

    # Renderizar el formulario y pasar el cliente al template
    return render(request, 'core/crear_transaccion.html', {'form': form, 'cliente': cliente})

@login_required
@verificar_permisos(es_administrador)
def editar_transaccion(request, transaccion_id):
    transaccion = get_object_or_404(Transacciones, id_transaccion=transaccion_id)
    clientes = Clientes.objects.all()
    if request.method == 'POST':
        transaccion.cliente = get_object_or_404(Clientes, id_cliente=request.POST['cliente'])
        transaccion.fecha = request.POST['fecha']
        transaccion.monto_recibido = request.POST.get('monto_recibido', 0)
        transaccion.monto_debitado = request.POST.get('monto_debitado', 0)
        transaccion.concepto = request.POST['concepto']
        transaccion.proveedor = request.POST['proveedor']
        transaccion.save()
        return redirect('listar_transacciones')
    return render(request, 'core/editar_transaccion.html', {'transaccion': transaccion, 'clientes': clientes})

@login_required
@verificar_permisos(es_administrador)
def eliminar_transaccion(request, transaccion_id):
    transaccion = get_object_or_404(Transacciones, id_transaccion=transaccion_id)
    if request.method == 'POST':
        transaccion.delete()
        return redirect('listar_transacciones')
    return render(request, 'core/eliminar_transaccion.html', {'transaccion': transaccion})

#Seccion Cobros
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user))
def listar_cobros(request):
    cobros = Cobros.objects.all().order_by('-fecha')  # Ordenar los cobros por fecha
    paginator = Paginator(cobros, 10)  # 10 cobros por página

    page_number = request.GET.get('page')  # Obtener el número de página de la URL
    page_obj = paginator.get_page(page_number)  # Obtener los objetos de la página actual

    return render(request, 'core/listar_cobros.html', {'page_obj': page_obj, 'cobros': page_obj})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def crear_cobro(request):
    clientes = Clientes.objects.all()
    if request.method == 'POST':
        cliente_id = request.POST['cliente']
        cliente = get_object_or_404(Clientes, id_cliente=cliente_id)
        fecha = request.POST['fecha']
        cantidad_autos = int(request.POST['cantidad_autos'] or 0)
        monto_pagar_usd = cantidad_autos * 50  # Calculado automáticamente en el HTML
        monto_usd = request.POST.get('monto_usd')  # Manual
        monto_gs = request.POST.get('monto_gs')    # Manual
        observaciones = request.POST.get('observaciones', '')

        # Validar y asignar valores para monto_usd y monto_gs
        try:
            monto_usd = float(monto_usd) if monto_usd else 0.0
        except ValueError:
            monto_usd = 0.0

        try:
            monto_gs = float(monto_gs) if monto_gs else 0.0
        except ValueError:
            monto_gs = 0.0

        Cobros.objects.create(
            cliente=cliente,
            fecha=fecha,
            cantidad_autos=cantidad_autos,
            monto_pagar_usd=monto_pagar_usd,  # Monto calculado
            monto_usd=monto_usd,  # Monto ingresado manualmente
            monto_gs=monto_gs,    # Monto ingresado manualmente
            observaciones=observaciones,
        )
        messages.success(request, "Cobro creado con éxito.")
        return redirect('listar_cobros')

    return render(request, 'core/crear_cobro.html', {'clientes': clientes})

@login_required
@verificar_permisos(es_administrador)
def editar_cobro(request, cobro_id):
    cobro = get_object_or_404(Cobros, id_cobro=cobro_id)
    clientes = Clientes.objects.all()
    if request.method == 'POST':
        cobro.cliente = get_object_or_404(Clientes, id_cliente=request.POST['cliente'])
        cobro.fecha = request.POST['fecha']
        cobro.monto_usd = request.POST.get('monto_usd', 0)
        cobro.monto_gs = request.POST.get('monto_gs', 0)
        cobro.observaciones = request.POST['observaciones']
        cobro.save()
        return redirect('listar_cobros')
    return render(request, 'core/editar_cobro.html', {'cobro': cobro, 'clientes': clientes})

@login_required
@verificar_permisos(es_administrador)
def eliminar_cobro(request, cobro_id):
    cobro = get_object_or_404(Cobros, id_cobro=cobro_id)
    if request.method == 'POST':
        cobro.delete()
        return redirect('listar_cobros')
    return render(request, 'core/eliminar_cobro.html', {'cobro': cobro})

@login_required
@verificar_permisos(es_administrador)
def resumen_cobros(request):
    # Obtener el año seleccionado desde la URL, por defecto el año actual
    anho = int(request.GET.get('anho', datetime.now().year))
    # Obtener todos los años disponibles en la base de datos
    anhos_disponibles = Cobros.objects.dates('fecha', 'year', order='DESC').distinct()
    # Filtrar los cobros por el año seleccionado
    cobros = Cobros.objects.filter(fecha__year=anho)
    # Inicializar resumen mensual
    resumen_mensual = []
    meses = [
        "ENERO", "FEBRERO", "MARZO", "ABRIL", "MAYO", "JUNIO",
        "JULIO", "AGOSTO", "SEPTIEMBRE", "OCTUBRE", "NOVIEMBRE", "DICIEMBRE"
    ]
    total_usd = 0
    total_gs = 0
    for i, mes in enumerate(meses, start=1):
        resumen = cobros.filter(fecha__month=i).aggregate(
            total_usd=Sum('monto_usd', default=0),
            total_gs=Sum('monto_gs', default=0)
        )
        total_usd += resumen['total_usd'] or 0
        total_gs += resumen['total_gs'] or 0
        resumen_mensual.append({
            'mes': mes,
            'total_usd': resumen['total_usd'] or 0,
            'total_gs': resumen['total_gs'] or 0,
        })
    context = {
        'anho': anho,
        'anhos_disponibles': [fecha.year for fecha in anhos_disponibles],
        'resumen_mensual': resumen_mensual,
        'total_usd': total_usd,
        'total_gs': total_gs,
    }
    return render(request, 'core/resumen_cobros.html', context)

#Importaciones 
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def crear_importacion(request):
    if request.method == 'POST':
        form = ImportacionesForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('listar_importaciones')
    else:
        form = ImportacionesForm()
    return render(request, 'core/crear_importacion.html', {'form': form})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_visitante(user) or es_vendedor(user))
def listar_importaciones(request):
    importaciones = Importaciones.objects.all().order_by('fecha')  # Ordenar por fecha
    paginator = Paginator(importaciones, 10)  # Mostrar 10 importaciones por página
    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)
    return render(request, 'core/listar_importaciones.html', {'page_obj': page_obj})

@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def editar_importacion(request, importacion_id):
    importacion = get_object_or_404(Importaciones, id_importacion=importacion_id)
    if request.method == 'POST':
        form = ImportacionesForm(request.POST, instance=importacion)
        if form.is_valid():
            form.save()
            messages.success(request, 'La importación ha sido actualizada con éxito.')
            return redirect('listar_importaciones')
    else:
        form = ImportacionesForm(instance=importacion)
    return render(request, 'core/editar_importacion.html', {'form': form, 'importacion': importacion})

@login_required
@verificar_permisos(lambda user: es_administrador(user))
def eliminar_importacion(request, importacion_id):
    importacion = get_object_or_404(Importaciones, id_importacion=importacion_id)
    if request.method == 'POST':
        importacion.delete()
        messages.success(request, 'La importación ha sido eliminada con éxito.')
        return redirect('listar_importaciones')
    return render(request, 'core/eliminar_importacion.html', {'importacion': importacion})

#Despacho de Vehiculos
@login_required
@verificar_permisos(lambda user: es_administrador(user) or es_vendedor(user))
def lista_vehiculos_por_despacho(request, despacho_id):
    # Obtener el despacho específico
    despacho = get_object_or_404(Importaciones, id_importacion=despacho_id)
    
    # Filtrar vehículos que tengan el número de despacho igual al despacho actual
    vehiculos = Vehiculos.objects.filter(numero_despacho=despacho)

    # Configurar la paginación
    paginator = Paginator(vehiculos, 10)  # Mostrar 10 vehículos por página
    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)

    return render(request, 'core/lista_vehiculos_por_despacho.html', {
        'despacho': despacho,
        'page_obj': page_obj
    })

#Cambiar Contraseña
@login_required
def cambiar_contrasena(request):
    if request.method == 'POST':
        form = CustomPasswordChangeForm(request.user, request.POST)
        if form.is_valid():
            user = form.save()
            update_session_auth_hash(request, user)  # Mantener la sesión después del cambio
            messages.success(request, 'Tu contraseña ha sido cambiada exitosamente.')
            print("✅ Contraseña actualizada correctamente")
            return redirect('dashboard')  # Redirigir al dashboard
        else:
            messages.error(request, '⚠️ Error: Corrige los datos antes de continuar.')
            print("❌ Error al actualizar la contraseña: ", form.errors)
    else:
        form = CustomPasswordChangeForm(request.user)

    return render(request, 'core/cambiar_contrasena.html', {'form': form})